//
//  ViewController.m
//  IndoorMapSDKDemo
//
//  Created by Macro on 16/4/21.
//  Copyright © 2016年 Macro. All rights reserved.
//

#import "ViewController.h"
#import <IndoorMapSDK/IndoorMapSDK.h>
#import <OnlineLocationSDK/OnlineLocationSDK.h>
#import <vector>

@interface ViewController ()<OIMMapViewDelegate, OIMRoutePlanningDelegate, OIMDownloadDelegate, IndoorLocationDelegate>


//KEY的获取请到 http://lbs.amap.com 申请
@property(nonatomic, strong)NSString* key;

//请在线申请建筑物数据，建筑物数据申请详情请参阅：http://lbs.amap.com/console/apply/
@property(nonatomic, strong)NSString* buildingId;


//室内地图对象
@property(nonatomic, strong)OIMMapView* imMapView;

//室内地图数据管理对象
@property(nonatomic, strong)OIMDataManager* imDataManager;

//室内地图路算对象
@property(nonatomic, strong)OIMRoutePlanning* imRoutePlanning;

//路算起点
@property(nonatomic, strong)OIMFeature* selectedFeature;

//路算起点
@property(nonatomic, strong)OIMFeature* routePlanningStart;

//室内定位
@property(nonatomic, strong)OnlineLocation* indoorLocation;

//定位按钮
@property(nonatomic, strong)UIImageView* locationButton;


//下载测试按钮
@property(nonatomic, strong)UIButton* downloadDataButton;
//路算测试按钮
@property(nonatomic, strong)UIButton* routePlanningButton;
//路算标志
@property(nonatomic)BOOL isRoutePlanning;
//搜索测试按钮
@property(nonatomic, strong)UIButton* searchKeywordsButton;


@property(nonatomic)BOOL enableOverlay;
@property(nonatomic, strong)NSMutableArray* annotations;
@property(nonatomic, strong)NSMutableArray* overlays;
@property(nonatomic)std::vector<OIMPoint>* clickedPoints;

@property(nonatomic, strong)UILabel* overlayView;
@property(nonatomic)CGPoint overlayViewCenter;

@property(nonatomic)BOOL autoSwitchFloor;

@end

@implementation ViewController

- (void)viewDidLoad
{
    [super viewDidLoad];

    //KEY的获取请到LBS.AMAP.COM 申请
    self.key = @"请输入自己的KEY";

    //请在线申请建筑物数据，建筑物数据申请详情请参阅：http://lbs.amap.com/console/apply/
    self.buildingId = @"请输入建筑物ID";

    self.buildingId = @"B000A856LJ";
    self.key = @"d0a84313b0dc20d2d3f0483bf52ae76b";

    //初始化室内地图
    self.imMapView = [[OIMMapView alloc]initWithFrame:self.view.bounds];
    //设置KEY
    self.imMapView.key = self.key;
    //设置delegate
    self.imMapView.delegate = self;
    //设置建筑物ID
    self.imMapView.buildingId = self.buildingId;
    //显示室内地图
    [self.view addSubview:self.imMapView];

    //添加定位按钮
    self.locationButton = [[UIImageView alloc]initWithImage:[self getImage:@"controlIcon/LocationNormal"]];
    self.locationButton.center = CGPointMake(25, self.view.bounds.size.height-25);
    [self.view addSubview:self.locationButton];
    self.locationButton.userInteractionEnabled = YES;
    [self.locationButton addGestureRecognizer:[[UITapGestureRecognizer alloc]initWithTarget:self action:@selector(singleTapLocationButton:)]];

    //添加下载按钮
    self.downloadDataButton = [[UIButton alloc]initWithFrame:CGRectMake(120, self.view.bounds.size.height-30, 50, 25)];
    [self.downloadDataButton setTitle:@"下载" forState:UIControlStateNormal];
    [self.downloadDataButton addTarget:self action:@selector(onClickedDownloadDataButton:) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:self.downloadDataButton];

    //添加路算按钮
    self.routePlanningButton = [[UIButton alloc]initWithFrame:CGRectMake(180, self.view.bounds.size.height-30, 50, 25)];
    [self.routePlanningButton setTitle:@"路算" forState:UIControlStateNormal];
    [self.routePlanningButton addTarget:self action:@selector(onClickedRoutePlanningButton:) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:self.routePlanningButton];

    //添加搜索按钮
    self.searchKeywordsButton = [[UIButton alloc]initWithFrame:CGRectMake(240, self.view.bounds.size.height-30, 50, 25)];
    [self.searchKeywordsButton setTitle:@"搜索" forState:UIControlStateNormal];
    [self.searchKeywordsButton addTarget:self action:@selector(onClickedSearchKeywordsButton:) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:self.searchKeywordsButton];

    //添加Overlay按钮
    self.searchKeywordsButton = [[UIButton alloc]initWithFrame:CGRectMake(300, self.view.bounds.size.height-30, 50, 25)];
    [self.searchKeywordsButton setTitle:@"Overlay" forState:UIControlStateNormal];
    [self.searchKeywordsButton addTarget:self action:@selector(onClickedOverlayButton:) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:self.searchKeywordsButton];

    self.clickedPoints = new std::vector<OIMPoint>();
    self.annotations = [[NSMutableArray alloc]init];
    self.overlays = [[NSMutableArray alloc]init];

    self.autoSwitchFloor = YES;
}

-(void)onClickedOverlayButton:(UIButton*)button
{
    self.enableOverlay = !self.enableOverlay;
    self.overlayView.hidden = !self.enableOverlay;
}


//室内地图加载成功事件
-(void)mapView:(OIMMapView *)mapView didFinishLoadingMap:(NSString *)buildingId floorNo:(int)floorNo
{
//    //提示地图加载成功
//    [self alert:@"室内地图" message:@"室内地图加载成功"];
    if([buildingId isEqualToString:@"有外延数据的建筑物"])
    {
        NSData* data = [NSData dataWithContentsOfFile:@"外延数据文件"];
        [self.imMapView loadExtensionData:data];
    }

    //初始化室内地图数据管理对象
    self.imDataManager = [[OIMDataManager alloc]init];
    //搜索名称中有"S"字的
    NSArray* features = [self.imDataManager searchByKeywords:@"S" floorNo:0];

    OIMStyle* style = [[OIMStyle alloc]initWithHeight:2 lineColor:[UIColor redColor] topColor:[UIColor greenColor] sideColor:[UIColor blueColor] sideColor2:[UIColor darkGrayColor] highlightColor:[UIColor yellowColor]];
//    [self.imMapView setStyle:style byFeatures:features];
    for (OIMFeature* feature in features)
    {
        [self.imMapView setStyle:style byId:feature.pid];
    }
}

-(void)mapView:(OIMMapView *)mapView didFailLoadingMap:(NSString *)buildingId floorNo:(int)floorNo withError:(NSError *)error
{
    //提示地图加载失败
    [self alert:@"室内地图加载失败" message:error.description];
}

//点击事件
-(void)mapView:(OIMMapView *)mapView didClickFeature:(OIMFeature *)feature
{
//    if(feature != nil)
//    {
//        OIMStyle* style = [[OIMStyle alloc]init];
//        style.topColor = [UIColor redColor];
//        [self.imMapView setStyle:style byId:feature.pid];
////        return;
//    }
    //清除所有的选择气泡显示
    [self.imMapView clearStatus:OIMFeatureStatus_Selected];
    //清除所有的高亮显示
    [self.imMapView setFeature:self.selectedFeature highlight:NO];
    //设置点击的对象显示气泡
    [self.imMapView setFeature:feature status:OIMFeatureStatus_Selected];
    //设置点击的对象高亮
    [self.imMapView setFeature:feature highlight:YES];
    self.selectedFeature = feature;
    //如果是在路算状态
    if(self.isRoutePlanning)
    {
        //如果还没有路算起点
        if(self.routePlanningStart==nil)
        {
            //清除路算结果
            [self.imMapView setShowRoutePlanning:NO];
            //设置路算起点
            self.routePlanningStart = feature;
            //设置路算起点图标
            [self.imMapView setFeature:feature status:OIMFeatureStatus_RouteStart];
            //清除路算终点图标
            [self.imMapView clearStatus:OIMFeatureStatus_RouteStop];
            //清除选择图标
            [self.imMapView clearStatus:OIMFeatureStatus_Selected];
            //提示设置路算终点
            [self alert:@"室内路算" message:@"请设置路算终点"];
        }
        else
        {
            //显示路算结果
            [self.imMapView setShowRoutePlanning:YES];
            //设置路算终点图标
            [self.imMapView setFeature:feature status:OIMFeatureStatus_RouteStop];
            //清除选择图标
            [self.imMapView clearStatus:OIMFeatureStatus_Selected];
            if(self.imRoutePlanning == nil)
            {
                //初始化室内路算对象
                self.imRoutePlanning = [[OIMRoutePlanning alloc]initWithDelegate:self];
                //设置路算对象的KEY
                self.imRoutePlanning.key = self.key;
            }
            //路算请求
            [self.imRoutePlanning requestRoutePlanning:self.buildingId fromPoiId:self.routePlanningStart.pid toPoiId:feature.pid];
//            [self.imRoutePlanning requestRoutePlanning:self.buildingId fromPoiId:@"GD0000080210400256" toPoiId:@"GD0000080210600349"];

            //清除路算起点
            self.routePlanningStart = nil;
            //清除路算标志
            self.isRoutePlanning = NO;
        }
    }

    if(self.overlayView != nil)
    {
        self.overlayView.text = feature.name;
    }
}

//点击下载按钮
-(void)onClickedDownloadDataButton:(UIButton*)button
{
    if(self.imDataManager == nil)
    {
        //初始化室内数据管理对象
        self.imDataManager = [[OIMDataManager alloc]init];
        //设置key，KEY的获取请到LBS.AMAP.COM 申请
        self.imDataManager.key = self.key;
        //监听下载事件
        self.imDataManager.downloadDelegate = self;
    }
    //请在线申请建筑物数据，建筑物数据申请详情请参阅：http://lbs.amap.com/console/apply/
    NSString* buildingId = @"请输入建筑物ID";
    buildingId = self.buildingId;
    //清除指定建筑物的室内数据
    [self.imDataManager removeBuilding:buildingId];
    //下载指定建筑物的室内数据
    [self.imDataManager downloadBuilding:buildingId];
}

//数据下载中事件
-(void)dataManager:(OIMDataManager *)dataManager didDownload:(NSString *)buildingId percent:(int)percent
{
    //提示下载百分比
    NSLog(@"下载百分比：%d%%\n", percent);
}

//数据下载成功事件
-(void)dataManager:(OIMDataManager *)dataManager didDownloadSuccess:(NSString *)buildingId updated:(BOOL)updated
{
    //提示下载结果
    [self alert:@"下载完成" message:[NSString stringWithFormat:@"%@%@", buildingId, (updated?@"有新数据下载":@"无新数据下载")]];
}

//数据下载失败事件
-(void)dataManager:(OIMDataManager *)dataManager didDownloadFailure:(NSString *)buildingId error:(NSError *)error
{
    //提示下载失败
    [self alert:@"下载失败" message:error.description];
}

//点击路算按钮
-(void)onClickedRoutePlanningButton:(UIButton*)button
{
    //设置路算标志
    self.isRoutePlanning = YES;
    //清除路算起点
    self.routePlanningStart = nil;
    //清除路算结果
    [self.imMapView setShowRoutePlanning:NO];
    //清除路算起点图标
    [self.imMapView clearStatus:OIMFeatureStatus_RouteStart];
    //清除路算终点图标
    [self.imMapView clearStatus:OIMFeatureStatus_RouteStop];
    //提示用户设置路算起点
    [self alert:@"室内路算" message:@"请设置路算起点"];
}



//路算成功事件
-(void)routePlanning:(OIMRoutePlanning *)routePlanning didRoutePlanningSuccess:(NSString *)buildingId routeFeature:(OIMFeature *)routeFeature
{
    //在地图上显示路算结果
    [self.imMapView setRoutePlanning:routeFeature];
    //路算结果
    [self alert:buildingId message:routeFeature.properties[@"json"]];
}

//路算失败事件
-(void)routePlanning:(OIMRoutePlanning *)routePlanning didRoutePlanningFailure:(NSString *)buildingId error:(NSError *)error
{
    //提示失败信息
    [self alert:@"路算失败" message:error.description];
}



//点击搜索按钮
-(void)onClickedSearchKeywordsButton:(UIButton*)button
{
    //初始化室内地图数据管理对象
    self.imDataManager = [[OIMDataManager alloc]init];
    //搜索名称中有"一"字的
    NSArray* features = [self.imDataManager searchByKeywords:@"H"];
    //搜索结果显示在地图上
    [self.imMapView setFeatures:features status:OIMFeatureStatus_Checked];
    if(features.count > 0)
    {
        //把第一个结果显示到地图的中央位置
        [self.imMapView setFeatureCenter:features[0]];
    }
    //把当前楼层的搜索结果（如果有）按合适的比例显示在屏幕上
    //[self.imMapView setFeaturesCenter:features];

    NSArray* categories = [NSArray arrayWithObjects:@61101, @50501, nil];
    features = [self.imDataManager searchByCategories:categories];
    [self.imMapView setFeatures:features status:OIMFeatureStatus_Checked];
}






//点击定位按钮，启动或停止室内定位
-(void)singleTapLocationButton:(UIGestureRecognizer*)gestureRecognizer
{
    if(self.indoorLocation == nil)
    {
        //启动室内定位
        self.indoorLocation = [[OnlineLocation alloc]initWithDelegate:self];
        self.indoorLocation.key = self.key;
        self.indoorLocation.buildingId = self.buildingId;

        [self.indoorLocation startLocation];

        self.locationButton.image = [self getImage:@"controlIcon/LocationClicked"];
        self.imMapView.showLocationPoint = YES;
//        self.imMapView.followingMode = OIMFollowingMode_FollowLocation;
    }
    else
    {
        //停止室内定位
        [self.indoorLocation stopLocation];
        self.indoorLocation = nil;

        self.locationButton.image = [self getImage:@"controlIcon/LocationNormal"];
        self.imMapView.showLocationPoint = NO;
    }
}

//定位成功事件
-(void)indoorLocation:(LocationBase *)indoorLocation didLocationResult:(LocationResult *)result
{
    if(![result.buildingId isEqualToString:self.imMapView.buildingId])
    {
        //不是当前建筑物，返回
        return;
    }
    if(result.floorId != self.imMapView.floorNo && self.autoSwitchFloor)
    {
        self.imMapView.floorNo = result.floorId;
        self.autoSwitchFloor = NO;
    }
    OIMPoint location = {result.longitude, result.latitude, (float)result.floorId};
    //在地图上显示定位点
    [self.imMapView setLocationPoint:location orientation:result.orientation accuracy:result.accuracy];
//    NSArray* results = [self.imDataManager search:location byDistance:10 limit:1];
//    [self alert:@"Found " message:[NSString stringWithFormat:@"%d results.", (int)results.count]];
}

//定位失败事件
-(void)indoorLocation:(LocationBase *)indoorLocation didLocationError:(NSString *)buildingId error:(NSError *)error
{
    //停止室内定位
    [self.indoorLocation stopLocation];
    self.indoorLocation = nil;
    self.locationButton.image = [self getImage:@"controlIcon/LocationNormal"];
    //提示失败信息
    [self alert:@"定位失败" message:error.description];
}


//提示信息
-(void)alert:(NSString*)title message:(NSString*)message
{
    //创建UIAlertController
    UIAlertController *alert = [UIAlertController alertControllerWithTitle:title message:message preferredStyle:UIAlertControllerStyleAlert];
    //添加确定按钮
    [alert addAction:[UIAlertAction actionWithTitle:@"确定" style:UIAlertActionStyleDefault handler:nil]];
    //显示Alert对话框
    [self presentViewController:alert animated:YES completion:nil];
}

//从资源中获取图片
-(UIImage*)getImage:(NSString *)imageName
{
    if(imageName == nil || imageName.length == 0)
    {
        return nil;
    }
    NSString* file = [self.imMapView.resourcePath stringByAppendingPathComponent:imageName];
    UIImage* image = [UIImage imageWithContentsOfFile:file];
    if(image == nil && ![imageName hasSuffix:@".png"])
    {
        file = [file stringByAppendingString:@".png"];
        image = [UIImage imageWithContentsOfFile:file];
    }
    return image;
}



//获取Annotation对应的View
-(OIMAnnotationView *)mapView:(OIMMapView *)mapView viewForAnnotation:(id<OIMAnnotation>)annotation
{
    int type = rand()%4;
    if(type > 2)
    {
        return [[OIMAnnotationView alloc]initWithAnnotation:annotation reuseIdentifier:@""];
    }
    else
    {
        OIMPinAnnotationView* view = [[OIMPinAnnotationView alloc]initWithAnnotation:annotation reuseIdentifier:@""];
        view.pinColor = (OIMPinAnnotationColor)type;
        return view;
    }
}

//获取Overlay对应的Renderer
-(OIMOverlayRenderer *)mapView:(OIMMapView *)mapView rendererForOverlay:(OIMOverlay *)overlay
{
    //线
    if([overlay isKindOfClass:OIMPolyline.class])
    {
        OIMPolyline* polyline = (OIMPolyline*)overlay;
        OIMPolylineRenderer* render = [[OIMPolylineRenderer alloc]initWithPolyline:polyline];
        render.lineWidth = 10;
//        render.fillColor = [UIColor yellowColor];
        render.strokeColor = [UIColor colorWithRed:rand()%255/255. green:rand()%255/255. blue:rand()%255/255. alpha:1];
        return render;
    }
    //面
    else if([overlay isKindOfClass:OIMPolygon.class])
    {
        OIMPolygon* polygon = (OIMPolygon*)overlay;
        OIMPolygonRenderer* render = [[OIMPolygonRenderer alloc]initWithPolygon:polygon];
        render.lineWidth = 10;
        render.fillColor = [UIColor redColor];
        render.strokeColor = [UIColor blueColor];
        return render;
    }
    return nil;
}

-(void)mapView:(OIMMapView *)mapView willClickFeature:(double)longitude latitude:(double)latitude
{
    //点击的坐标
    self.overlayViewCenter = CGPointMake(longitude, latitude);
    if(self.enableOverlay)
    {
        //初始化overlayView
        if(self.overlayView == nil)
        {
            self.overlayView = [[UILabel alloc]initWithFrame:CGRectMake(0, 0, 100, 30)];
            [self.imMapView addSubview:self.overlayView];
            self.overlayView.backgroundColor = [[UIColor alloc]initWithWhite:0 alpha:0];
        }
        //移动overlayView
        self.overlayView.center = [self.imMapView convertCoordinate:self.overlayViewCenter];

        //初始化一个标注
        OIMPointAnnotation* annotation = [[OIMPointAnnotation alloc]init];
        //设置标注属性
        annotation.coordinate = OIMPointMake(longitude, latitude, self.imMapView.floorNo);
        //添加标注
        [self.imMapView addAnnotation:annotation];
        [self.annotations addObject:annotation];

        [self.imMapView setLocationPoint:annotation.coordinate orientation:0 accuracy:1];
//        self.imMapView.followingMode = OIMFollowingMode_FollowLocation;
        //记录点信息
        self.clickedPoints->insert(self.clickedPoints->begin(), annotation.coordinate);
        if(self.clickedPoints->size()>rand()%10+3)
        {
            OIMOverlay* overlay = nil;
            //初始化一条线
            if(rand()%2)
            {
                overlay = [OIMPolyline polylineWithPoints:self.clickedPoints->data() count:self.clickedPoints->size()];
            }
            //初始化一个面
            else
            {
                overlay = [OIMPolygon polygonWithPoints:self.clickedPoints->data() count:self.clickedPoints->size()];
            }
            //添加Overlay
            [self.imMapView addOverlay:overlay];
            [self.overlays addObject:overlay];

            self.clickedPoints->clear();
            //清除标注
            [self.imMapView removeAnnotations:self.annotations];
            [self.annotations removeAllObjects];
        }
    }
}


-(void)mapView:(OIMMapView *)mapView didFinishRenderingMap:(NSString *)buildingId floorNo:(int)floorNo
{
    //移动overlayView
    if(self.overlayView != nil)
    {
        self.overlayView.center = [self.imMapView convertCoordinate:self.overlayViewCenter];
    }
}


@end
